import DeviceModel from '../../models/device'
import NodeModel from '../../models/node'
import PropertyModel from '../../models/property'
import PropertyHistoryModel from '../../models/property-history'
import AutomationScriptModel from '../../models/automation-script'
import Device from '../infrastructure/device'
import Node from '../infrastructure/node'
import Property from '../infrastructure/property'

/**
 * This funcion bridges the infrastructure to the database
 */
export function bridgeInfrastructureToDatabase ({$deps, infrastructure}) {
  let databaseQueue = Promise.resolve()

  infrastructure.on('automationUpdated', async () => {
    const automation = infrastructure.getAutomation()
    if (!automation.model) {
      automation.model = await AutomationScriptModel.forge({ script: automation.script, blockly_xml: automation.xml }).save()
    } else {
      await automation.model.set({
        script: automation.script,
        xml: automation.xml
      }).save()
    }
  })

  infrastructure.on('newDevice', function onNewDevice (device) {
    databaseQueue = databaseQueue.then(() => {
      $deps.log.debug(`inserting new device ${device.id} into DB`)
      return DeviceModel.forge({
        id: device.id,
        name: device.name,
        online: device.online,
        local_ip: device.localIp,
        mac: device.mac,
        stats_signal: device.getStatProperty('signal'),
        stats_uptime: device.getStatProperty('uptime'),
        stats_interval_in_seconds: device.getStatProperty('interval'),
        fw_name: device.getFirmwareProperty('name'),
        fw_version: device.getFirmwareProperty('version'),
        fw_checksum: device.getFirmwareProperty('checksum'),
        implementation: device.implementation
      }).save(null, { method: 'insert' })
    }).then((model) => {
      device.model = model
    })
  })

  infrastructure.on('newNode', function onNewNode (node) {
    databaseQueue = databaseQueue.then(() => {
      $deps.log.debug(`inserting new node ${node.id} into DB`)
      return NodeModel.forge({
        device_id: node.device.id,
        device_node_id: node.id,
        name: node.type,
        type: node.type,
        properties: node.propertiesDefinition
      }).save()
    }).then((model) => {
      node.model = model
    })
  })

  infrastructure.on('newProperty', function onNewProperty (property) {
    databaseQueue = databaseQueue.then(() => {
      $deps.log.debug(`inserting new property ${property.id} into DB`)
      return PropertyModel.forge({
        node_id: property.node.model.id,
        node_property_id: property.id,
        settable: property.settable
      }).save()
    }).then((model) => {
      property.model = model

      if (!property.value) return
      return PropertyHistoryModel.forge({
        property_id: property.model.id,
        value: property.value,
        date: new Date()
      }).save()
    })
  })

  infrastructure.on('update', function onUpdate (update) {
    switch (update.entity.constructor) {
      case Device:
        if (!update.entity.model) return
        databaseQueue = databaseQueue.then(() => {
          $deps.log.debug(`updating device ${update.entity.id} in DB`)
          return update.entity.model.set({
            name: update.entity.name,
            online: update.entity.online,
            local_ip: update.entity.localIp,
            stats_signal: update.entity.getStatProperty('signal'),
            stats_uptime: update.entity.getStatProperty('uptime'),
            stats_interval_in_seconds: update.entity.getStatProperty('interval'),
            fw_name: update.entity.getFirmwareProperty('name'),
            fw_version: update.entity.getFirmwareProperty('version'),
            fw_checksum: update.entity.getFirmwareProperty('checksum')
          }).save()
        })

        break
      case Node:
        if (!update.entity.model) return
        databaseQueue = databaseQueue.then(() => {
          $deps.log.debug(`updating node ${update.entity.id} in DB`)
          return update.entity.model.set({
            type: update.entity.type,
            properties: update.entity.propertiesDefinition
          }).save()
        })

        break
      case Property:
        if (!update.entity.model) return
        databaseQueue = databaseQueue.then(() => {
          $deps.log.debug(`updating property ${update.entity.id} in DB`)
          if (!update.entity.value) return
          return PropertyHistoryModel.forge({
            property_id: update.entity.model.id,
            value: update.entity.value,
            date: new Date()
          }).save()
        })
    }
  })
}
